@@ -74,7 +74,7 @@ struct section_map *section_map)
     char *strings;
     enum bool force_extern_reloc;
     struct undefined_map *undefined_map;
-    struct merged_symbol *merged_symbol, **hash_pointer;
+    struct merged_symbol *merged_symbol;
     struct section_map *local_map, *pair_local_map;
     struct relocation_info *reloc, *pair_reloc;
     struct scattered_relocation_info *sreloc, *spair_reloc;
@@ -339,14 +339,13 @@ struct section_map *section_map)
 		    if((nlists[symbolnum].n_type & N_TYPE) == N_SECT &&
 		       (cur_obj->section_maps[nlists[symbolnum].n_sect-1].
 			s->flags & SECTION_TYPE) == S_COALESCED){
-			hash_pointer = lookup_symbol(strings +
+			merged_symbol = lookup_symbol(strings +
 					     nlists[symbolnum].n_un.n_strx);
-			if(hash_pointer == NULL){
+			if(merged_symbol->name_len == 0){
 			    fatal("internal error, in hppa_reloc() failed to "
 			          "lookup coalesced symbol %s", strings +
 				  nlists[symbolnum].n_un.n_strx);
 			}
-			merged_symbol = *hash_pointer;
 		    }
 		    else{
 			if((nlists[symbolnum].n_type & N_EXT) != N_EXT ||
@@ -497,6 +496,14 @@ struct section_map *section_map)
 		    }
 		    local_map = &(cur_obj->section_maps[r_symbolnum - 1]);
 		    local_map->output_section->referenced = TRUE;
+		    if(local_map->s->flags & S_ATTR_DEBUG){
+			error_with_cur_obj("illegal reference to debug section,"
+			    " from non-debug section (%.16s,%.16s) via "
+			    "relocation entry (%lu) to section (%.16s,%.16s)",
+			    section_map->s->segname, section_map->s->sectname,
+			    i, local_map->s->segname, local_map->s->sectname);
+			return;
+		    }
 		    pair_local_map = NULL;
 		    if(r_type == HPPA_RELOC_SECTDIFF ||
 		       r_type == HPPA_RELOC_HI21_SECTDIFF ||
@@ -504,6 +511,16 @@ struct section_map *section_map)
 			pair_local_map =
 			    &(cur_obj->section_maps[pair_r_symbolnum - 1]);
 			pair_local_map->output_section->referenced = TRUE;
+			if(pair_local_map->s->flags & S_ATTR_DEBUG){
+			    error_with_cur_obj("illegal reference to debug "
+				"section, from non-debug section (%.16s,%.16s) "
+				"via relocation entry (%lu) to section (%.16s,"
+				"%.16s)", section_map->s->segname,
+				section_map->s->sectname, i,
+				pair_local_map->s->segname,
+				pair_local_map->s->sectname);
+			    return;
+			}
 		    }
 		    if(local_map->nfine_relocs == 0 && 
 		       (pair_local_map == NULL ||
