@@ -24,9 +24,13 @@
 #include <mach/m68k/thread_status.h>
 #undef MACHINE_THREAD_STATE	/* need to undef these to avoid warnings */
 #undef MACHINE_THREAD_STATE_COUNT
+#undef THREAD_STATE_NONE
+#undef VALID_THREAD_STATE_FLAVOR
 #include <mach/ppc/thread_status.h>
 #undef MACHINE_THREAD_STATE	/* need to undef these to avoid warnings */
 #undef MACHINE_THREAD_STATE_COUNT
+#undef THREAD_STATE_NONE
+#undef VALID_THREAD_STATE_FLAVOR
 #include <mach/m88k/thread_status.h>
 #include <mach/i860/thread_status.h>
 #include <mach/i386/thread_status.h>
@@ -77,6 +81,7 @@ struct load_command *load_commands)
     struct routines_command_64 *rc64;
     struct twolevel_hints_command *hints;
     struct prebind_cksum_command *cs;
+    struct uuid_command *uuid;
     unsigned long flavor, count, nflavor;
     char *p, *state;
 
@@ -918,6 +923,15 @@ struct load_command *load_commands)
 		}
 		break;
 
+	    case LC_UUID:
+		uuid = (struct uuid_command *)lc;
+		if(uuid->cmdsize != sizeof(struct uuid_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_UUID command %lu has incorrect cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
 	    default:
 		error("in swap_object_headers(): malformed load commands "
 		      "(unknown load command %lu)", i);
@@ -1326,6 +1340,11 @@ struct load_command *load_commands)
 		cs = (struct prebind_cksum_command *)lc;
 		swap_prebind_cksum_command(cs, target_byte_sex);
 		break;
+
+	    case LC_UUID:
+		uuid = (struct uuid_command *)lc;
+		swap_uuid_command(uuid, target_byte_sex);
+		break;
 	    }
 
 	    lc = (struct load_command *)((char *)lc + l.cmdsize);
